@@ -11,10 +11,10 @@
 # under the License.
 
 import collections
+from datetime import datetime
 import logging
 import os.path
 
-import six
 import yaml
 
 from reno import scanner
@@ -30,8 +30,7 @@ def get_cache_filename(conf):
 class Loader(object):
     "Load the release notes for a given repository."
 
-    def __init__(self, conf,
-                 ignore_cache=False):
+    def __init__(self, conf, ignore_cache=False):
         """Initialize a Loader.
 
         The versions are presented in reverse chronological order.
@@ -56,7 +55,9 @@ class Loader(object):
         self._cache = None
         self._scanner = None
         self._scanner_output = None
+        self._tags_to_dates = None
         self._cache_filename = get_cache_filename(conf)
+        self._encoding = conf.options['encoding']
 
         self._load_data()
 
@@ -68,18 +69,37 @@ class Loader(object):
 
         if (not self._ignore_cache) and cache_file_exists:
             LOG.debug('loading cache file %s', self._cache_filename)
-            with open(self._cache_filename, 'r') as f:
+
+            with open(self._cache_filename, 'r', encoding=self._encoding) as f:
                 self._cache = yaml.safe_load(f.read())
-                # Save the cached scanner output to the same attribute
-                # it would be in if we had loaded it "live". This
-                # simplifies some of the logic in the other methods.
-                self._scanner_output = collections.OrderedDict(
-                    (n['version'], n['files'])
-                    for n in self._cache['notes']
-                )
+
+        if self._cache:
+            # Save the cached scanner output to the same attribute
+            # it would be in if we had loaded it "live". This
+            # simplifies some of the logic in the other methods.
+            self._scanner_output = collections.OrderedDict(
+                (n['version'], n['files'])
+                for n in self._cache['notes']
+            )
+            self._tags_to_dates = collections.OrderedDict(
+                (n['version'], n['date'])
+                for n in self._cache['dates']
+            )
         else:
             self._scanner = scanner.Scanner(self._config)
             self._scanner_output = self._scanner.get_notes_by_version()
+            self._tags_to_dates = self._scanner.get_version_dates()
+
+    def close(self):
+        """Close any files opened by this loader."""
+        if self._scanner is not None:
+            self._scanner.close()
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.close()
 
     @property
     def versions(self):
@@ -90,6 +110,13 @@ class Loader(object):
         "Return data about the files that should go into a given version."
         return self._scanner_output[version]
 
+    def get_version_date(self, version):
+        "Return release data for a version."
+        if version in self._tags_to_dates.keys():
+            date = datetime.fromtimestamp(self._tags_to_dates[version])
+            return date.strftime("%Y-%m-%d")
+        return "Unknown"
+
     def parse_note_file(self, filename, sha):
         """Return the data structure encoded in the note file.
 
@@ -105,38 +132,59 @@ class Loader(object):
 
         cleaned_content = {}
 
+        if not isinstance(content, dict):
+            LOG.warning(
+                '%s does not appear to be structured as a YAML mapping. '
+                'Did you forget a top-level key?',
+                filename,
+            )
+            raise ValueError(
+                f'{filename} does not appear to be structured as a YAML '
+                f'mapping. Did you forget a top-level key?'
+            )
+
+        valid_section_names = {
+            section.name for section in self._config.sections
+        }
         for section_name, section_content in content.items():
             if section_name == self._config.prelude_section_name:
-                if not isinstance(section_content, six.string_types):
+                if not isinstance(section_content, str):
                     LOG.warning(
-                        ('The %s section of %s '
-                         'does not parse as a single string. '
-                         'Is the YAML input escaped properly?') %
-                        (self._config.prelude_section_name, filename),
+                        'The %s section of %s does not parse as a single '
+                        'string. Is the YAML input escaped properly?',
+                        section_name, filename,
                     )
             else:
-                if isinstance(section_content, six.string_types):
+                if section_name not in valid_section_names:
+                    # TODO(stephenfin): Make this an error in a future release
+                    LOG.warning(
+                        'The %s section of %s is not a recognized section. '
+                        'It should be one of: %s. '
+                        'This will be an error in a future release.',
+                        section_name, filename,
+                        ', '.join(valid_section_names),
+                    )
+                if isinstance(section_content, str):
                     # A single string is OK, but wrap it with a list
                     # so the rest of the code can treat the data model
                     # consistently.
                     section_content = [section_content]
                 elif not isinstance(section_content, list):
                     LOG.warning(
-                        ('The %s section of %s '
-                         'does not parse as a string or list of strings. '
-                         'Is the YAML input escaped properly?') % (
-                             section_name, filename),
+                        'The %s section of %s does not parse as a string or '
+                        'list of strings. Is the YAML input escaped properly?',
+                        section_name, filename,
                     )
                 else:
                     for item in section_content:
-                        if not isinstance(item, six.string_types):
+                        if not isinstance(item, str):
                             LOG.warning(
-                                ('The item %r in the %s section of %s '
-                                 'parses as a %s instead of a string. '
-                                 'Is the YAML input escaped properly?'
-                                 ) % (item, section_name,
-                                      filename, type(item)),
+                                'The item %r in the %s section of %s parses '
+                                'as a %s instead of a string. '
+                                'Is the YAML input escaped properly?',
+                                item, section_name, filename, type(item),
                             )
+
             cleaned_content[section_name] = section_content
 
         return cleaned_content
