@@ -12,10 +12,13 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from unittest import mock
+
 import fixtures
 import io
-import mock
+import os
 
+from reno import config
 from reno import create
 from reno.tests import base
 
@@ -68,8 +71,9 @@ class TestCreate(base.TestCase):
         args.from_template = self._create_user_template('i-am-a-user-template')
         args.slug = 'theslug'
         args.edit = False
-        conf = mock.Mock()
+        conf = mock.create_autospec(config.Config)
         conf.notespath = self.tmpdir
+        conf.options = {'allow_subdirectories': False, 'encoding': None}
         with mock.patch('sys.stdout', new=io.StringIO()) as fake_out:
             create.create_cmd(args, conf)
         filename = self._get_file_path_from_output(fake_out.getvalue())
@@ -82,10 +86,36 @@ class TestCreate(base.TestCase):
         args.from_template = 'some-unexistent-file.yaml'
         args.slug = 'theslug'
         args.edit = False
-        conf = mock.Mock()
+        conf = mock.create_autospec(config.Config)
         conf.notespath = self.tmpdir
+        conf.options = {'allow_subdirectories': False, 'encoding': None}
         self.assertRaises(ValueError, create.create_cmd, args, conf)
 
+    def test_create_from_user_template_fails_because_path_separator(self):
+        args = mock.Mock()
+        args.from_template = self._create_user_template('i-am-a-user-template')
+        args.slug = 'the' + os.sep + 'slug'
+        args.edit = False
+        conf = mock.create_autospec(config.Config)
+        conf.notespath = self.tmpdir
+        conf.options = {'allow_subdirectories': False, 'encoding': None}
+        self.assertRaises(ValueError, create.create_cmd, args, conf)
+
+    def test_create_from_template_with_path_separator_allowed(self):
+        args = mock.Mock()
+        args.from_template = self._create_user_template('i-am-a-user-template')
+        args.slug = 'the' + os.sep + 'slug'
+        args.edit = False
+        conf = mock.create_autospec(config.Config)
+        conf.notespath = self.tmpdir
+        conf.options = {'allow_subdirectories': True, 'encoding': None}
+        with mock.patch('sys.stdout', new=io.StringIO()) as fake_out:
+            create.create_cmd(args, conf)
+        filename = self._get_file_path_from_output(fake_out.getvalue())
+        with open(filename, 'r') as f:
+            body = f.read()
+        self.assertEqual('i-am-a-user-template', body)
+
     def test_edit(self):
         self.useFixture(fixtures.EnvironmentVariable('EDITOR', 'myeditor'))
         with mock.patch('subprocess.call') as call_mock:
